#define AMDA_DEBUG
#include <amda.h>
#include <iostream>
#include <string>
#include <set>

using namespace AMDA;
using namespace std;

#if !defined(TEST_SEPARATED) && !defined(TEST_STRUCTURED) && \
    !defined(TEST_DELTA_CHECK)
#error TEST_* not defined.
#endif

#if defined(TEST_SEPARATED) || defined(TEST_STRUCTURED)
#ifdef TEST_SEPARATED
#define STORAGE Standard::SeparatedStorage
#else
#define STORAGE Standard::StructuredStorage
#endif
typedef Standard::Traits<char, size_t, unsigned int, STORAGE> TR;
typedef DoubleArray<TR> DA;
typedef AMDA::Standard::SortedKeySource<TR> SortedKeySource;
typedef TR::ArrayBody ArrayBody;
typedef AMDA::Standard::FileDrain<TR> FileDrain;
typedef AMDA::Standard::FileSource<TR> FileSource;
typedef const char *KeyType;
#else // TEST_DELTA_CHECK
typedef DeltaCheck::Traits<size_t, unsigned int> TR;
typedef DoubleArray<TR> DA;
typedef AMDA::DeltaCheck::SortedKeySource<TR> SortedKeySource;
typedef TR::ArrayBody ArrayBody;
typedef AMDA::DeltaCheck::FileDrain<TR> FileDrain;
typedef AMDA::DeltaCheck::FileSource<TR> FileSource;
typedef const AMDA::U8 *KeyType;
#endif

typedef set<string> KeySet;

int
main()
{
	Status rv;
	KeySet keyset;

	// read keys from stdin - separated by whitespaces.
	while (!cin.eof()) {
		string key;
		cin >> key;
		if (key.size())
			keyset.insert(key);
	}

	// build sorted array of keys.
	KeyType *keys = new KeyType [keyset.size()];
	size_t *keylens = new size_t [keyset.size()];
	int count = 0;
	for (KeySet::const_iterator i=keyset.begin(); i!=keyset.end(); ++i) {
		keylens[count] = i->size();
		keys[count] = (KeyType)i->c_str();
		count++;
	}

	// build double array.
	DA da;
	rv = da.build(SortedKeySource(count, keys, keylens));
	if (rv) {
		cerr << "error=" << (int)rv << endl;
		return 1;
	}

	// test dump/restore to/from file.
	cout << "dump." << endl;
	rv = da.dump(FileDrain("test2.da"));
	if (rv) {
		cerr << "error=" << (int)rv << endl;
		return 1;
	}
	cout << "restore.\n" << endl;
	rv = da.build(FileSource("test2.da"));
	if (rv) {
		cerr << "error=" << (int)rv << endl;
		return 1;
	}

	// print array contents.
	const ArrayBody &ab = da.array_body();
	cout << "[0] base=" << ab.base(0, 0) << "(node)" << endl;
	for (size_t i=1; i<ab.num_entries(); i++) {
		if (ab.is_inuse(i, 0)) {
			cout << "[" << i << "] check=" << ab.check(i, 0)
			     << ", base=" << ab.base(i, 0)
			     << "(" << (ab.check(i, 0)==i?"leaf":"node")
			     << ")" << endl;
		}
	}
	cout << endl;

	// self common prefix search.
	cout << "self common prefix search:" << endl;
	for (int i=0; i<count; i++) {
		cout << "  " << keys[i] << ":" << endl;
		DA::Walker w(da, keys[i], keylens[i]);
		do {
			if (w.is_leaf()) {
				cout << "    [" << w.get_leaf_id() << "] "
				     << keys[w.get_leaf_id()] << endl;
			}
		} while (!(rv = w()));
		if (rv != S_BREAK) {
			cout << "  not match - strange..." << endl;
		}
	}

	return 0;
}

